@@ -30,6 +30,7 @@ import codecs
 import contextlib
 import functools
 import os.path
+import sys
 import threading
 import types
 import weakref
@@ -714,7 +715,7 @@ def merge_adjacent_actions_with_language(tokens):
 
 
 def get_bom_encoding(data):
-    """Get the BOM (Byte Order Mark) of data, if any.
+    """Get the BOM (Byte Order Mark) of bytes ``data``, if any.
 
     A two-tuple is returned (encoding, data). If the data starts with a BOM
     mark, its encoding is determined and the BOM mark is stripped off.
@@ -758,8 +759,21 @@ def unroll(obj):
 
     """
     if type(obj) in (tuple, list):
-        for i in obj:
-            yield from unroll(i)
+        stack = []
+        gen = iter(obj)
+        while True:
+            for obj in gen:
+                if type(obj) in (tuple, list):
+                    stack.append(gen)
+                    gen = iter(obj)
+                    break
+                else:
+                    yield obj
+            else:
+                if stack:
+                    gen = stack.pop()
+                else:
+                    break
     else:
         yield obj
 
@@ -780,3 +794,29 @@ def tokens(nodes, reverse=False):
             yield from n.tokens(reverse)
 
 
+def language_sister_class(language, template, base, try_parents=False):
+    """Find a ``language`` sister class in the same module, with a name that
+    matches the ``template``, and which is a subclass of ``base``.
+
+    If ``try_parents`` is True, the parent classes of the language class are
+    checked if a sister class is not found.
+
+    The template string should contain ``{}``, which is replaced with the
+    language's class name. Returns None if no sister class is defined.
+    Example::
+
+        >>> from parce.util import language_sister_class
+        >>> from parce.lang.css import Css
+        >>> from parce.transform import Transform
+        >>> language_sister_class(Css, "{}Transform", Transform)
+        <class 'parce.lang.css.CssTransform'>
+
+    """
+    langs = language.mro()[:-2] if try_parents else [language]
+    for l in langs:
+        module = sys.modules[l.__module__]
+        name = template.format(l.__name__)
+        cls = getattr(module, name, None)
+        if isinstance(cls, type) and issubclass(cls, base):
+            return cls
+
